<?php
namespace wcf\acp\form;
use wcf\data\category\Category;
use wcf\data\category\CategoryNodeTree;
use wcf\data\jwt\JWTService;
use wcf\data\jwt\JWTServiceAction;
use wcf\data\jwt\JWTServiceEditor;
use wcf\form\AbstractForm;
use wcf\system\database\util\PreparedStatementConditionBuilder;
use wcf\system\exception\UserInputException;
use wcf\system\language\I18nHandler;
use wcf\system\WCF;
use wcf\util\FileUtil;
use wcf\util\ImageUtil;
use wcf\util\StringUtil;

/**
 * Shows the jwt service add form.
 * 
 * @author	Franz Reiter (Howaner)
 * @copyright	2019 Franz Reiter
 * @license	GNU Lesser General Public License <http://opensource.org/licenses/lgpl-license.php>
 * @package	de.howaner.jwt
 */
class JWTServiceAddForm extends AbstractForm {
	/**
	 * @inheritDoc
	 */
	public $activeMenuItem = 'wcf.acp.menu.link.jwtservice';
	
	/**
	 * @inheritDoc
	 */
	public $templateName = 'jwtServiceAdd';
	
	/**
	 * @inheritDoc
	 */
	public $neededPermissions = ['admin.configuration.canManageJwtServices'];
	
	/**
	 * Jwt service name
	 * @var string
	 */
	public $name = '';

	/**
	 * Used algorithm for jwt signature
	 * @var string
	 */
	public $algorithm = '';

	/**
	 * Jwt service secret
	 * @var string
	 */
	public $secret = '';

	/**
	 * Redirect url after jwt request
	 * @var string
	 */
	public $redirectURL = '';

	/**
	 * Lifetime of the token in seconds (used for expiration date)
	 * Default: 1 Week
	 * @var int
	 */
	public $lifetime = 604800;
	
	/**
	 * @inheritDoc
	 */
	public function assignVariables() {
		parent::assignVariables();
		
		WCF::getTPL()->assign([
			'action' => 'add',
			'name' => $this->name,
			'algorithm' => $this->algorithm,
			'supportedAlgorithms' => JWTService::getSupportedAlgorithms(),
			'secret' => $this->secret,
			'redirectURL' => $this->redirectURL,
			'lifetime' => $this->lifetime
		]);
	}
	
	/**
	 * @inheritDoc
	 */
	public function readData() {
		parent::readData();
	}
	
	/**
	 * @inheritDoc
	 */
	public function readParameters() {
		parent::readParameters();
	}
	
	/**
	 * @inheritDoc
	 */
	public function readFormParameters() {
		parent::readFormParameters();
		
		if (isset($_POST['name'])) $this->name = StringUtil::trim($_POST['name']);
		if (isset($_POST['algorithm'])) $this->algorithm = $_POST['algorithm'];
		if (isset($_POST['secret'])) $this->secret = $_POST['secret'];
		if (isset($_POST['redirectURL'])) $this->redirectURL = StringUtil::trim($_POST['redirectURL']);
		if (isset($_POST['lifetime']) && is_numeric($_POST['lifetime'])) $this->lifetime = intval($_POST['lifetime']);
	}
	
	/**
	 * @inheritDoc
	 */
	public function save() {
		parent::save();
		
		$this->objectAction = new JWTServiceAction([], 'create', [
			'data' => array_merge($this->additionalFields, [
				'name' => $this->name,
				'algorithm' => $this->algorithm,
				'secret' => $this->secret,
				'redirectURL' => $this->redirectURL,
				'lifetime' => $this->lifetime
			])
		]);
		$this->objectAction->executeAction();
		$returnValues = $this->objectAction->getReturnValues();
		
		// reset values
		$this->name = '';
		$this->algorithm = '';
		$this->secret = '';
		$this->redirectURL = '';
		$this->lifetime = 604800;
		
		$this->saved();
		
		// show success message
		WCF::getTPL()->assign('success', true);
	}
	
	/**
	 * @inheritDoc
	 */
	public function validate() {
		parent::validate();
		
		if (empty($this->name))
			throw new UserInputException('name');
		if (empty($this->algorithm) || !in_array($this->algorithm, JWTService::getSupportedAlgorithms()))
			throw new UserInputException('algorithm');
		if (empty($this->secret))
			throw new UserInputException('secret');
		if (empty($this->redirectURL))
			throw new UserInputException('redirectURL');
		if (empty($this->lifetime))
			throw new UserInputException('lifetime');
	}
}

